@@ -2,52 +2,82 @@ import os
 
 from devassistant import exceptions
 from devassistant import settings
-from devassistant.command_helpers import ClHelper, PathHelper
 
 class AssistantBase(object):
     """WARNING: if assigning subassistants in __init__, make sure to override it
     in subclass, so that it doesn't get inherited!"""
     # Some informations about assistant
-    # These are mandatory:
+    # These should all be present:
     name = 'base'
     fullname = 'Base'
-
-    # These are optional:
     description = ''
+    role = 'creator'
     args = []
-    repo = []
+    source_file = ''
 
-    template_dir = os.path.join(os.path.dirname(__file__), 'templates')
+    template_dir = os.path.join(os.path.dirname(__file__), 'data', 'templates')
 
-    # don't override these, used internally
-    _dot_devassistant_path = None
-    # we don't use this currently, so let's keep it commented so that we don't depend on jinja for no reason
-    # _jinja_env = jinja2.Environment(loader=jinja2.FileSystemLoader(template_dir))
+    def get_subassistant_classes(self):
+        """Return list of classes that are subassistants of this assistant.
 
-    def get_subassistants(self):
+        Override in subclasses representing assistants written in Python
+
+        Returns:
+            list of classes that are subassistants of this assistant
+        """
         return []
 
-    def get_subassistant_chain(self):
-        if not '_chain' in dir(self):
-            subas_list = []
-            if 'get_subassistants' in vars(self.__class__): # only non-inherited get_subassistants
-                for subas in self.get_subassistants():
-                    subas_list.append(subas().get_subassistant_chain())
-            self._chain = (self, subas_list)
-        return self._chain
+    def get_subassistants(self):
+        """Return list of instantiated subassistants.
+
+        Usually, this needs not be overriden in subclasses, you should just override
+        get_subassistant_classes
+
+        Returns:
+            list of instantiated subassistants
+        """
+        if not hasattr(self, '_subassistants'):
+            self._subassistants = []
+            # we want to know, if type(self) defines 'get_subassistant_classes',
+            # we don't want to inherit it from superclass (would cause recursion)
+            if 'get_subassistant_classes' in vars(type(self)):
+                for a in self.get_subassistant_classes():
+                    self._subassistants.append(a())
+        return self._subassistants
+
+    def get_subassistant_tree(self):
+        """Returns a tree-like structure representing the assistant hierarchy going down
+        from this assistant to leaf assistants.
+
+        For example: [(<This Assistant>,
+                       [(<Subassistant 1>, [...]),
+                        (<Subassistant 2>, [...])]
+                      )]
+        Returns:
+            a tree-like structure (see above) representing assistant hierarchy going down
+            from this assistant to leaf assistants
+        """
+        if not '_tree' in dir(self):
+            subassistant_tree = []
+            subassistants = self.get_subassistants()
+            for subassistant in subassistants:
+                subassistant_tree.append(subassistant.get_subassistant_tree())
+            self._tree = (self, subassistant_tree)
+        return self._tree
 
     def get_selected_subassistant_path(self, **kwargs):
-        """Recursively searches self._chain - has format of (Assistant: [list_of_subassistants]) -
+        """Recursively searches self._tree - has format of (Assistant: [list_of_subassistants]) -
         for specific path from first to last selected subassistants.
+
         Args:
             kwargs: arguments containing names of the given assistants in form of
             subassistant_0 = 'name', subassistant_1 = 'another_name', ...
         Returns:
-            List of subassistants objects from chain sorted from first to last.
+            list of subassistants objects from tree sorted from first to last
         """
         path = [self]
         previous_subas_list = None
-        currently_searching = self.get_subassistant_chain()[1]
+        currently_searching = self.get_subassistant_tree()[1]
 
         # len(path) - 1 always points to next subassistant_N, so we can use it to control iteration
         while settings.SUBASSISTANT_N_STRING.format(len(path) - 1) in kwargs and \
@@ -84,7 +114,7 @@ class AssistantBase(object):
         Errors should not be logged, only returned.
 
         Returns:
-            List of errors as strings (empty list with no errors).
+            list of errors as strings (empty list with no errors)
         """
         return []
 
@@ -104,9 +134,3 @@ class AssistantBase(object):
             devassistant.exceptions.RunException containing the error message
         """
         pass
-
-    def _git_create_repo(self, path, gitignore, **kwargs):
-        PathHelper.cp(gitignore, path)
-        ClHelper.run_command('git init')
-        ClHelper.run_command('git add .')
-        ClHelper.run_command('git commit -m "Initial commit."')
