@@ -1,20 +1,104 @@
 import copy
+import functools
 import logging
 import os
-import sys
 
+from devassistant import argument
 from devassistant import assistant_base
 from devassistant import exceptions
 from devassistant.assistants.command_formatter import CommandFormatter
 from devassistant.assistants.commands import run_command
 from devassistant.logger import logger
+from devassistant import yaml_loader
 from devassistant import yaml_snippet_loader
+from devassistant import package_managers
+
+def needs_fully_loaded(method):
+    """Wraps all publicly callable methods of YamlAssistant. If the assistant was loaded
+    from cache, this decorator will fully load it first time a publicly callable method
+    is used.
+    """
+    @functools.wraps(method)
+    def inner(self, *args, **kwargs):
+        if not self.fully_loaded:
+            self.parsed_yaml = yaml_loader.YamlLoader.load_yaml_by_path(self.path).popitem()[1]
+            self.fully_loaded = True
+        return method(self, *args, **kwargs)
+
+    return inner
 
 class YamlAssistant(assistant_base.AssistantBase):
-    _dependencies = {}
+    def __init__(self, name, parsed_yaml, path, template_dir, fully_loaded=True, role='creator'):
+        self.name = name
+        self.parsed_yaml = parsed_yaml
+        self.path = path
+        self.template_dir = template_dir
+        self.fully_loaded = fully_loaded
+        self.role = role
+        self.stop_flag = False
 
-    _run = []
+    @property
+    def parsed_yaml(self):
+        return self._parsed_yaml
 
+    @parsed_yaml.setter
+    def parsed_yaml(self, value):
+        self._parsed_yaml = value
+
+        self.template_dir = value.get('template_dir', self.template_dir)
+        self.fullname = value.get('fullname', '')
+        self.description = value.get('description', '')
+        self.args = self._construct_args(value.get('args', {}))
+
+        self._files = value.get('files', {})
+        self._logging = value.get('logging', [])
+        # set _run and _dependencies as empty in case assistant doesn't have them at all
+        self._dependencies = value.get('dependencies', [])
+        self._run = value.get('run', [])
+        # handle more dependencies* and run* sections
+        for k, v in value.items():
+            if k.startswith('run') or k.startswith('dependencies'):
+                setattr(self, '_{0}'.format(k), v)
+        self._pre_run = value.get('pre_run', [])
+        self._post_run = value.get('post_run', [])
+
+    @needs_fully_loaded
+    def assert_fully_loaded(self):
+        return True
+
+    def _construct_args(self, struct):
+        args = []
+        for arg_name, arg_params in struct.items():
+            use_snippet = arg_params.pop('snippet', None)
+            if use_snippet:
+                # if snippet is used, take this parameter from snippet and update
+                # it with current arg_params, if any
+                try:
+                    problem = None
+                    snippet = yaml_snippet_loader.YamlSnippetLoader.get_snippet_by_name(use_snippet)
+                    arg_params = dict(snippet.args.pop(arg_name), **arg_params)
+                except exceptions.SnippetNotFoundException as e:
+                    problem = 'Couldn\'t expand argument {arg} in assistant {a}: ' + str(e)
+                except KeyError as e: # snippet doesn't have the requested argument
+                    problem = 'Couldn\'t find argument {arg} in snippet {snip} wanted by assistant {a}.'
+
+                if problem:
+                    logger.warning(problem.format(snip=use_snippet,
+                                                  arg=arg_name,
+                                                  a=self.name))
+                    continue
+
+                # this works much like snippet.args.pop(arg_name).update(arg_params),
+                # but unlike it, this actually returns the updated dict
+
+            arg = argument.Argument(arg_name, *arg_params.pop('flags'), **arg_params)
+            args.append(arg)
+        return args
+
+    def get_subassistants(self):
+        return self._subassistants
+
+    @needs_fully_loaded
     def proper_kwargs(self, **kwargs):
         """Returns kwargs possibly updated with values from .devassistant
         file, when appropriate."""
@@ -26,6 +110,7 @@ class YamlAssistant(assistant_base.AssistantBase):
             kwargs = new_kwargs
         return kwargs
 
+    @needs_fully_loaded
     def logging(self, **kwargs):
         kwargs = self.proper_kwargs(**kwargs)
         for l in self._logging:
@@ -46,6 +131,7 @@ class YamlAssistant(assistant_base.AssistantBase):
             else:
                 logger.warning('Unknown logger type {0}, ignoring.'.format(handler_type))
 
+    @needs_fully_loaded
     def dependencies(self, **kwargs):
         """Returns all dependencies of this assistant with regards to specified kwargs.
 
@@ -90,7 +176,7 @@ class YamlAssistant(assistant_base.AssistantBase):
                         deps.extend(self._dependencies_section(section, **kwargs))
                     else:
                         logger.warning('Couldn\'t find dependencies section "{0}", in snippet {1}, skipping.'.format(dep_list.split('.')))
-                elif dep_type in ['rpm']: # handle known types of deps the same, just by appending to "deps" list
+                elif dep_type in package_managers.managers.keys(): # handle known types of deps the same, just by appending to "deps" list
                     deps.append({dep_type: dep_list})
                 elif dep_type.startswith('if'):
                     possible_else = None
@@ -109,6 +195,7 @@ class YamlAssistant(assistant_base.AssistantBase):
 
         return deps
 
+    @needs_fully_loaded
     def run(self, **kwargs):
         kwargs = self.proper_kwargs(**kwargs)
         if self.role == 'modifier':
@@ -125,16 +212,20 @@ class YamlAssistant(assistant_base.AssistantBase):
                     break
         else:
             to_run = self._get_section_to_run(section='run', kwargs_override=True, **kwargs)
-        if 'pre_run' in dir(self):
-            self._run_one_section(self.pre_run, kwargs)
+
+        kwargs['__assistant__'] = self
+        if self._pre_run:
+            self._run_one_section(self._pre_run, kwargs)
         self._run_one_section(to_run, kwargs)
-        if 'post_run' in dir(self):
-            self._run_one_section(self.post_run, kwargs)
+        if self._post_run:
+            self._run_one_section(self._post_run, kwargs)
 
     def _run_one_section(self, section, kwargs):
         skip_else = False
 
         for i, command_dict in enumerate(section):
+            if self.stop_flag:
+                break
             for comm_type, comm in command_dict.items():
                 if comm_type.startswith('call'):
                     # calling workflow:
@@ -149,17 +240,20 @@ class YamlAssistant(assistant_base.AssistantBase):
                         continue
 
                     if self._is_snippet_call(comm, **kwargs):
-                        # we're calling a snippet => add files to kwargs
+                        # we're calling a snippet => add files and template_dir to kwargs
                         snippet = yaml_snippet_loader.YamlSnippetLoader.get_snippet_by_name(comm.split('.')[0])
 
                         if '__files__' not in kwargs:
                             kwargs['__files__'] = []
+                            kwargs['__template_dir__'] = []
                         kwargs['__files__'].append(snippet.get_files_section())
+                        kwargs['__template_dir__'].append(snippet.get_template_dir())
 
                     self._run_one_section(sect, copy.deepcopy(kwargs))
 
                     if self._is_snippet_call(comm, **kwargs):
                         kwargs['__files__'].pop()
+                        kwargs['__template_dir__'].pop()
                 elif comm_type.startswith('$'):
                     # intentionally pass kwargs as dict, not as keywords
                     try:
@@ -204,7 +298,8 @@ class YamlAssistant(assistant_base.AssistantBase):
                     kwargs['__scls__'].pop()
                 else:
                     files = kwargs['__files__'][-1] if kwargs.get('__files__', None) else self._files
-                    run_command(comm_type, CommandFormatter.format(comm_type, comm, self.template_dir, files, **kwargs), **kwargs)
+                    template_dir = kwargs['__template_dir__'][-1] if kwargs.get('__template_dir__', None) else self.template_dir
+                    run_command(comm_type, CommandFormatter.format(comm_type, comm, template_dir, files, **kwargs), **kwargs)
 
     def _is_snippet_call(self, cmd_call, **kwargs):
         return not (cmd_call == 'self' or cmd_call.startswith('self.'))
@@ -230,7 +325,7 @@ class YamlAssistant(assistant_base.AssistantBase):
             try:
                 control_var = self._get_var_name(for_parts[1])
                 error = None
-            except exceptions.YamlSyntaxError as e:
+            except exceptions.YamlSyntaxError:
                 pass # specify the error more?
 
         if error:
@@ -292,7 +387,7 @@ class YamlAssistant(assistant_base.AssistantBase):
                     section = snippet.get_run_section(section_name) if snippet else None
                 else:
                     section = snippet.get_dependencies_section(section_name) if snippet else None
-            except exceptions.SnippetNotFoundException as e:
+            except exceptions.SnippetNotFoundException:
                 section = None
 
         return section
@@ -389,3 +484,9 @@ class YamlAssistant(assistant_base.AssistantBase):
             raise exceptions.YamlSyntaxError('Not a valid expression: ' + expression)
 
         return (success if not invert_success else not success, output)
+
+    @needs_fully_loaded
+    def stop(self):
+        """ This function is used for stopping devassistant from GUI
+        """
+        self.stop_flag = True
