@@ -2,6 +2,7 @@ import argparse
 import getpass
 import logging
 import os
+import signal
 import subprocess
 import sys
 import tempfile
@@ -13,9 +14,8 @@ from devassistant import settings
 
 class ClHelper(object):
     @classmethod
-    def run_command(cls, cmd_str, log_level=logging.DEBUG, scls=[]):
+    def run_command(cls, cmd_str, log_level=logging.DEBUG, scls=[], ignore_sigint=False):
         """Runs a command from string, e.g. "cp foo bar" """
-
         # format for scl execution if needed
         cmd_str = cls.format_for_scls(cmd_str, scls)
         logger.log(log_level, cmd_str, extra={'event_type': 'cmd_call'})
@@ -33,11 +33,13 @@ class ClHelper(object):
         stdin_pipe = None
         stdout_pipe = subprocess.PIPE
         stderr_pipe = subprocess.STDOUT
+        preexec_fn = cls.ignore_sigint if ignore_sigint else None
         proc = subprocess.Popen(cmd_str,
                                 stdin=stdin_pipe,
                                 stdout=stdout_pipe,
                                 stderr=stderr_pipe,
-                                shell=True)
+                                shell=True,
+                                preexec_fn=preexec_fn)
         stdout = []
         while proc.poll() == None:
             output = proc.stdout.readline().strip().decode('utf8')
@@ -64,96 +66,9 @@ class ClHelper(object):
                                                                                    scls=' '.join(scls))
         return cmd_str
 
-class RPMHelper(object):
-    c_rpm = 'rpm'
-
     @classmethod
-    def rpm_q(cls, rpm_name):
-        try:
-            # if we install by e.g. virtual provide, then rpm -q foo will fail
-            # therefore we always use rpm -q --whatprovides foo
-            return ClHelper.run_command(' '.join([cls.c_rpm,
-                                                  '-q',
-                                                  '--whatprovides',
-                                                  '"' + rpm_name.strip() + '"']))
-        except exceptions.ClException:
-            return False
-
-    @classmethod
-    def is_rpm_installed(cls, rpm_name):
-        logger.info('Checking for presence of {0}...'.format(rpm_name), extra={'event_type': 'dep_check'})
-
-        found_rpm = cls.rpm_q(rpm_name)
-        if found_rpm:
-            logger.info('Found {0}'.format(found_rpm), extra={'event_type': 'dep_found'})
-        else:
-            logger.info('Not found, will install', extra={'event_type': 'dep_not_found'})
-        return found_rpm
-
-    @classmethod
-    def was_rpm_installed(cls, rpm_name):
-        # TODO: handle failure
-        found_rpm = cls.rpm_q(rpm_name)
-        logger.info('Installed {0}'.format(found_rpm), extra={'event_type': 'dep_installed'})
-        return found_rpm
-
-
-class YUMHelper(object):
-    c_yum = 'yum'
-
-    @classmethod
-    def resolve(cls, *args):
-        logger.info('Resolving dependencies ...')
-        import yum
-        y = yum.YumBase()
-        y.setCacheDir(tempfile.mkdtemp())
-        for arg in args:
-            if arg.startswith('@'):
-                y.selectGroup(arg[1:])
-            else:
-                pkg = y.returnPackageByDep(arg)
-                y.install(pkg)
-        y.resolveDeps()
-        logger.debug('Installing/Updating:')
-        to_install = []
-        for pkg in y.tsInfo.getMembers():
-            to_install.append(pkg.po.ui_envra)
-            logger.debug(pkg.po.ui_envra)
-
-        return to_install
-
-    @classmethod
-    def install(cls, *args):
-        try:
-            to_install = cls.resolve(*args)
-        except BaseException as e:
-            logger.error('Failed to resolve dependencies: {exc}'.format(exc=e))
-            return False
-        ret = DialogHelper.ask_for_confirm_with_message(prompt='Install following packages?',
-                                                        message='\n'.join(sorted(to_install)))
-        if ret is False:
-            return False
-
-        cmd = ['pkexec', cls.c_yum, '-y', 'install']
-        quoted_pkgs = map(lambda pkg: '"{pkg}"'.format(pkg=pkg), to_install)
-        cmd.extend(quoted_pkgs)
-        try:
-            ClHelper.run_command(' '.join(cmd), log_level=logging.INFO)
-            return args
-        except exceptions.ClException:
-            return False
-
-    @classmethod
-    def is_group_installed(cls, group):
-        logger.info('Checking for presence of group {0}...'.format(group))
-
-        output = ClHelper.run_command(' '.join([cls.c_yum, 'group', 'list', '"{0}"'.format(group)]))
-        if 'Installed Groups' in output:
-            logger.info('Found %s', group)
-            return True
-
-        logger.info('Not found')
-        return False
+    def ignore_sigint(cls):
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
 
 class PathHelper(object):
     c_cp = 'cp'
@@ -203,7 +118,8 @@ class DialogHelper(object):
                 cls.user_override_helper = values
                 setattr(namespace, self.dest, values)
 
-        arg = argument.Argument(settings.UI_FLAG,
+        arg = argument.Argument(settings.UI_FLAG[2:],
+                                settings.UI_FLAG,
                                 action=UIAction,
                                 required=False,
                                 help=help)
